@@ -19,6 +19,7 @@ import java.lang.reflect.Method;
 import java.util.HashMap;
 import java.util.Map;
 
+import org.gnu.glib.EventType;
 import org.gnu.gnome.event.AppBarListener;
 import org.gnu.gnome.event.ClientListener;
 import org.gnu.gnome.event.ColorPickerListener;
@@ -42,7 +43,6 @@ import org.gnu.gtk.event.ExposeListener;
 import org.gnu.gtk.event.FocusListener;
 import org.gnu.gtk.event.FontButtonListener;
 import org.gnu.gtk.event.GtkEvent;
-import org.gnu.gtk.event.GtkEventType;
 import org.gnu.gtk.event.ItemListener;
 import org.gnu.gtk.event.KeyListener;
 import org.gnu.gtk.event.LifeCycleListener;
@@ -100,7 +100,7 @@ abstract class ListenerDelegate {
 		try {
 			Widget source = (Widget)event.getSource();
 			//System.err.println("fireEvent: source:"+source);
-			GtkEventType typeNeeded = source.getEventType(signal);
+			EventType typeNeeded = source.getEventType(signal);
 			//System.err.println("fireEvent: typeNeeded:"+typeNeeded);
 			if (event.getType() == typeNeeded) {
 				Class[] paramTypes = handler.getParameterTypes();
@@ -140,7 +140,13 @@ abstract class ListenerDelegate {
 		throws ClassNotFoundException, NoSuchMethodException, InvocationTargetException {
 		try {
 			Method handler = getHandlerMethod(owner, handlerName);
-			Class delegateClass = (Class)delegateMap.get(listenerClass);
+			Class delegateClass = (Class)delegateClassMap.get(listenerClass);
+			if (delegateClass == null) {
+			    String listenerClassName = listenerClass.getName();
+			    String delegateClassName = (String) delegateMap.get(listenerClassName);
+			    delegateClass = Class.forName(delegateClassName);
+			    delegateClassMap.put(listenerClass, delegateClass);
+			}
 			if (delegateClass == null)
 				throw new ClassNotFoundException("No libglade delegate for " + listenerClass.getName());
 
@@ -158,47 +164,48 @@ abstract class ListenerDelegate {
 
 	// Map GTK and GNOME listener interfaces to their respective delegates.
 	private static Map delegateMap = new HashMap();
+	private static Map delegateClassMap = new HashMap();
 	static {
-		delegateMap.put(AdjustmentListener.class, AdjustmentDelegate.class);
-		delegateMap.put(ButtonListener.class, ButtonDelegate.class);
-		delegateMap.put(CalendarListener.class, CalendarDelegate.class);
-		delegateMap.put(CellRendererTextListener.class, CellRendererTextDelegate.class);
-		delegateMap.put(CheckMenuItemListener.class, CheckMenuItemDelegate.class);
-		delegateMap.put(ColorButtonListener.class, ColorButtonDelegate.class);
-		delegateMap.put(ComboBoxListener.class, ComboBoxDelegate.class);
-		delegateMap.put(ContainerListener.class, ContainerDelegate.class);
-		delegateMap.put(DialogListener.class, DialogDelegate.class);
-		delegateMap.put(EntryListener.class, EntryDelegate.class);
-		delegateMap.put(ExpanderListener.class, ExpanderDelegate.class);
-		delegateMap.put(ExposeListener.class, ExposeDelegate.class);
-		delegateMap.put(FocusListener.class, FocusDelegate.class);
-		delegateMap.put(FontButtonListener.class, FontButtonDelegate.class);
-		delegateMap.put(ItemListener.class, ItemDelegate.class);
-		delegateMap.put(KeyListener.class, KeyDelegate.class);
-		delegateMap.put(LifeCycleListener.class, LifeCycleDelegate.class);
-		delegateMap.put(MenuItemListener.class, MenuItemDelegate.class);
-		delegateMap.put(MouseListener.class, MouseDelegate.class);
-		delegateMap.put(MouseMotionListener.class, MouseMotionDelegate.class);
-		delegateMap.put(NotebookListener.class, NotebookDelegate.class);
-		delegateMap.put(OptionMenuListener.class, OptionMenuDelegate.class);
-		delegateMap.put(RangeListener.class, RangeDelegate.class);
-		delegateMap.put(ScaleListener.class, ScaleDelegate.class);
-		delegateMap.put(SpinListener.class, SpinDelegate.class);
-		delegateMap.put(TextBufferListener.class, TextBufferDelegate.class);
-		delegateMap.put(ToggleListener.class, ToggleDelegate.class);
-		delegateMap.put(ToolBarListener.class, ToolBarDelegate.class);
-		delegateMap.put(ToolButtonListener.class, ToolButtonDelegate.class);
-		delegateMap.put(TreeModelListener.class, TreeModelDelegate.class);
-		delegateMap.put(TreeSelectionListener.class, TreeSelectionDelegate.class);
-		delegateMap.put(TreeViewColumnListener.class, TreeViewColumnDelegate.class);
-		delegateMap.put(TextViewListener.class, TextViewDelegate.class);
+		delegateMap.put("org.gnu.gtk.event.AdjustmentListener", "org.gnu.glade.AdjustmentDelegate");
+		delegateMap.put("org.gnu.gtk.event.ButtonListener", "org.gnu.glade.ButtonDelegate");
+		delegateMap.put("org.gnu.gtk.event.CalendarListener", "org.gnu.glade.CalendarDelegate");
+		delegateMap.put("org.gnu.gtk.event.CellRendererTextListener", "org.gnu.glade.CellRendererTextDelegate");
+		delegateMap.put("org.gnu.gtk.event.CheckMenuItemListener", "org.gnu.glade.CheckMenuItemDelegate");
+		delegateMap.put("org.gnu.gtk.event.ColorButtonListener", "org.gnu.glade.ColorButtonDelegate");
+		delegateMap.put("org.gnu.gtk.event.ComboBoxListener", "org.gnu.glade.ComboBoxDelegate");
+		delegateMap.put("org.gnu.gtk.event.ContainerListener", "org.gnu.glade.ContainerDelegate");
+		delegateMap.put("org.gnu.gtk.event.DialogListener", "org.gnu.glade.DialogDelegate");
+		delegateMap.put("org.gnu.gtk.event.EntryListener", "org.gnu.glade.EntryDelegate");
+		delegateMap.put("org.gnu.gtk.event.ExpanderListener", "org.gnu.glade.ExpanderDelegate");
+		delegateMap.put("org.gnu.gtk.event.ExposeListener", "org.gnu.glade.ExposeDelegate");
+		delegateMap.put("org.gnu.gtk.event.FocusListener", "org.gnu.glade.FocusDelegate");
+		delegateMap.put("org.gnu.gtk.event.FontButtonListener", "org.gnu.glade.FontButtonDelegate");
+		delegateMap.put("org.gnu.gtk.event.ItemListener", "org.gnu.glade.ItemDelegate");
+		delegateMap.put("org.gnu.gtk.event.KeyListener", "org.gnu.glade.KeyDelegate");
+		delegateMap.put("org.gnu.gtk.event.LifeCycleListener", "org.gnu.glade.LifeCycleDelegate");
+		delegateMap.put("org.gnu.gtk.event.MenuItemListener", "org.gnu.glade.MenuItemDelegate");
+		delegateMap.put("org.gnu.gtk.event.MouseListener", "org.gnu.glade.MouseDelegate");
+		delegateMap.put("org.gnu.gtk.event.MouseMotionListener", "org.gnu.glade.MouseMotionDelegate");
+		delegateMap.put("org.gnu.gtk.event.NotebookListener", "org.gnu.glade.NotebookDelegate");
+		delegateMap.put("org.gnu.gtk.event.OptionMenuListener", "org.gnu.glade.OptionMenuDelegate");
+		delegateMap.put("org.gnu.gtk.event.RangeListener", "org.gnu.glade.RangeDelegate");
+		delegateMap.put("org.gnu.gtk.event.ScaleListener", "org.gnu.glade.ScaleDelegate");
+		delegateMap.put("org.gnu.gtk.event.SpinListener", "org.gnu.glade.SpinDelegate");
+		delegateMap.put("org.gnu.gtk.event.TextBufferListener", "org.gnu.glade.TextBufferDelegate");
+		delegateMap.put("org.gnu.gtk.event.ToggleListener", "org.gnu.glade.ToggleDelegate");
+		delegateMap.put("org.gnu.gtk.event.ToolBarListener", "org.gnu.glade.ToolBarDelegate");
+		delegateMap.put("org.gnu.gtk.event.ToolButtonListener", "org.gnu.glade.ToolButtonDelegate");
+		delegateMap.put("org.gnu.gtk.event.TreeModelListener", "org.gnu.glade.TreeModelDelegate");
+		delegateMap.put("org.gnu.gtk.event.TreeSelectionListener", "org.gnu.glade.TreeSelectionDelegate");
+		delegateMap.put("org.gnu.gtk.event.TreeViewColumnListener", "org.gnu.glade.TreeViewColumnDelegate");
+		delegateMap.put("org.gnu.gtk.event.TextViewListener", "org.gnu.glade.TextViewDelegate");
 
-		delegateMap.put(AppBarListener.class, AppBarDelegate.class);
-		delegateMap.put(ClientListener.class, ClientDelegate.class);
-		delegateMap.put(ColorPickerListener.class, ColorPickerDelegate.class);
-		delegateMap.put(DateEditListener.class, DateEditDelegate.class);
-		delegateMap.put(DruidPageChangeListener.class, DruidPageDelegate.class);
-		delegateMap.put(DruidPageSetupListener.class, DruidPageDelegate.class);
-		delegateMap.put(FontPickerListener.class, FontPickerDelegate.class);
+		delegateMap.put("org.gnu.gnome.AppBarListener", "org.gnu.glade.AppBarDelegate");
+		delegateMap.put("org.gnu.gnome.ClientListener", "org.gnu.glade.ClientDelegate");
+		delegateMap.put("org.gnu.gnome.ColorPickerListener", "org.gnu.glade.ColorPickerDelegate");
+		delegateMap.put("org.gnu.gnome.DateEditListener", "org.gnu.glade.DateEditDelegate");
+		delegateMap.put("org.gnu.gnome.DruidPageChangeListener", "org.gnu.glade.DruidPageDelegate");
+		delegateMap.put("org.gnu.gnome.DruidPageSetupListener", "org.gnu.glade.DruidPageDelegate");
+		delegateMap.put("org.gnu.gnome.FontPickerListener", "org.gnu.glade.FontPickerDelegate");
 	}
 }
